package com.example.conexionasincrona;

import androidx.annotation.NonNull;
import androidx.appcompat.app.AppCompatActivity;

import android.app.ProgressDialog;
import android.content.Context;
import android.content.DialogInterface;
import android.os.AsyncTask;
import android.os.Bundle;
import android.util.Log;
import android.view.View;
import android.widget.Toast;

import com.example.conexionasincrona.databinding.ActivityMainBinding;

import java.io.IOException;
import java.net.HttpURLConnection;
import java.net.MalformedURLException;
import java.net.URL;

import okhttp3.Call;
import okhttp3.Callback;
import okhttp3.OkHttpClient;
import okhttp3.Request;
import okhttp3.Response;
import okhttp3.ResponseBody;

public class MainActivity extends AppCompatActivity implements View.OnClickListener {
    private ActivityMainBinding binding;
    long inicio, fin;
    TareaAsincrona tareaAsincrona;
    URL url;

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.activity_main);

        binding = ActivityMainBinding.inflate(getLayoutInflater());
        View view = binding.getRoot();
        setContentView(view);

        binding.button.setOnClickListener(this);
    }

    @Override
    public void onClick(View view) {

        try {
            url = new URL(binding.editText.getText().toString());
            if (binding.switch1.isChecked()) {
                //descarga usando OkHttp
                descargaOkHTTP(url);
            } else
                // descarga usando AsyncTask y Java.net
                descarga(url);
        } catch (MalformedURLException e) {
            e.printStackTrace();
            mostrarError(e.getMessage());
        } catch (IOException ex) {
            mostrarError(ex.getMessage());
        }
    }

    private void descargaOkHTTP(URL web) {
        inicio = System.currentTimeMillis();
        final OkHttpClient client = new OkHttpClient();
        Request request = new Request.Builder()
                .url(web)
                .build();
        client.newCall(request).enqueue(new Callback() {
            @Override
            public void onResponse(@NonNull Call call, @NonNull Response response) throws IOException {
                try (ResponseBody responseBody = response.body()) {
                    if (!response.isSuccessful()) {
                        //throw new IOException("Unexpected code " + response);
                        mostrarRespuesta("Unexpected code: " + "<br>" + response);
                    } else {
                        // Read data on the worker thread
                        final String responseData = response.body().string();
                        //Toast.makeText(getApplicationContext(), datos, Toast.LENGTH_SHORT).show();
                        mostrarRespuesta(responseData);
                    }
                }
            }
            @Override
            public void onFailure(@NonNull Call call, @NonNull IOException e) {
                Log.e("Error: ", e.getMessage());
                mostrarRespuesta("Fallo: " + e.getMessage());
            }
        });
    }

    private void mostrarRespuesta(String message) {
        fin = System.currentTimeMillis();
        // Run view-related code back on the main thread
        MainActivity.this.runOnUiThread(new Runnable() {
            @Override
            public void run() {
                binding.webView.loadDataWithBaseURL(String.valueOf(url), message, "text/html", "UTF-8", null);
                binding.textView.setText("Duración: " + String.valueOf(fin - inicio) + " milisegundos");
            }
        });
    }

    private void descarga(URL url) {
        inicio = System.currentTimeMillis();
        tareaAsincrona = new TareaAsincrona(this);
        tareaAsincrona.execute(url);
        binding.textView.setText("Descargando la página");
    }

    private void mostrarError(String mensaje) {
        Toast.makeText(this, mensaje, Toast.LENGTH_SHORT).show();
    }

    public class TareaAsincrona extends AsyncTask<URL, Void, Resultado> {
        private ProgressDialog progreso;
        private Context context;

        public TareaAsincrona(Context context) {
            this.context = context;
        }

        protected void onPreExecute() {
            progreso = new ProgressDialog(context);
            progreso.setProgressStyle(ProgressDialog.STYLE_SPINNER);
            progreso.setMessage("Conectando . . .");
            progreso.setCancelable(true);
            progreso.setOnCancelListener(new DialogInterface.OnCancelListener() {
                public void onCancel(DialogInterface dialog) {
                    TareaAsincrona.this.cancel(true);
                }
            });
            progreso.show();
        }

        protected Resultado doInBackground(URL... cadena) {
            Resultado resultado;

            try {
                // operaciones en el hilo secundario
                resultado = Conexion.conectarJava(cadena[0]);
            } catch (IOException e) {
                Log.e("HTTP", e.getMessage(), e);
                resultado = new Resultado();
                // https://developer.mozilla.org/es/docs/Web/HTTP/Status
                // https://httpstatuses.com/
                resultado.setCodigo(500);
                resultado.setMensaje(e.getMessage());
                //cancel(true);
            }
            return resultado;
        }

        protected void onPostExecute(Resultado resultado) {
            progreso.dismiss();
            // mostrar el resultado
            fin = System.currentTimeMillis();
            if (resultado.getCodigo() == HttpURLConnection.HTTP_OK)
                binding.webView.loadDataWithBaseURL(String.valueOf(url), resultado.getContenido(), "text/html", "UTF-8", null);
            else {
                mostrarError(resultado.getMensaje());
                binding.webView.loadDataWithBaseURL(String.valueOf(url), resultado.getMensaje(), "text/html", "UTF-8", null);
            }
            binding.textView.setText("Duración: " + String.valueOf(fin - inicio) + " milisegundos");
        }

        protected void onCancelled() {
            progreso.dismiss();
            // mostrar cancelación
            //Toast.makeText(context, "Cancelado", Toast.LENGTH_SHORT).show();
            mostrarError("Cancelado");
        }
    }
} //MainActivity